package cn.com.qmth.scancentral.task.thread;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.qmth.boot.core.concurrent.service.ConcurrentService;
import com.qmth.boot.core.exception.ParameterException;

import cn.com.qmth.scancentral.enums.LockType;
import cn.com.qmth.scancentral.service.StudentService;
import cn.com.qmth.scancentral.vo.asynctask.ExamStatusResetTaskVo;

public class ExamStatusResetThread implements Runnable {

    protected static Logger log = LoggerFactory.getLogger(ExamStatusResetThread.class);

    private ExamStatusResetTaskVo vo;

    private StudentService studentService;

    private ConcurrentService concurrentService;

    public ExamStatusResetThread(ExamStatusResetTaskVo vo, StudentService studentService,
            ConcurrentService concurrentService) {
        this.vo = vo;
        this.studentService = studentService;
        this.concurrentService = concurrentService;
    }

    @Override
    public void run() {
        try {
            if (concurrentService.getLock(LockType.EXAM_STATUS_RESET + "-" + vo.getExamId()).tryLock()) {
                studentService.resetExamStatus(vo);
                vo.setTotalCount(vo.getProgressCount());
            }
        } catch (ParameterException e) {
            vo.setErrMsg(e.getMessage());
            vo.setSuccess(false);
        } catch (Exception e) {
            log.error("exam status reset error", e);
            vo.setErrMsg("缺考校验重新生成失败");
            vo.setSuccess(false);
        } finally {
            concurrentService.getLock(LockType.EXAM_STATUS_RESET + "-" + vo.getExamId()).unlock();
        }
    }

}
